//+------------------------------------------------------------------+
//|                                                    DRAW_FIBO.mq4 |
//|                                Copyright 2015, Daniel & Friends. |
//|                      https://www.facebook.com/bluetradingschool/ |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Daniel & Friends."
#property link      "https://www.facebook.com/bluetradingschool/"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property script_show_inputs

input double fibo0;
input double fibo100;
input color fbColor = clrRed;
double levelValues[] = {0, 0.236, 0.382, 0.618, 1.00, 1.618, 4.236, 2.618};
string fiboName;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
//---
   fiboName = StringConcatenate("Fibo",DoubleToString(fibo0),DoubleToStr(fibo100));
   int counted_bars = IndicatorCounted();
   /*
   double price1, price2;
   if(fibo0 > fibo100){
      price1 = getFiboLevel(fibo0, fibo100, 1, -1);
      price2 = fibo0;
   }else{
      
      price1 = getFiboLevel(fibo100, fibo0, 1, 1);
      price2 = fibo0;
   }
   Print(fiboName);
   
   if(!FiboLevelsCreate(0,fiboName,0,TimeCurrent(),price1, iTime(NULL, PERIOD_MN1, Bars),price2,fbColor,
      STYLE_DASHDOT,1,false,false,true,false,0))
     {
      return(0);
     }
     */
   
   for(int i=0; i < ArraySize(levelValues); i++){
      double linePrice;
      
      if(fibo0 > fibo100){
         linePrice = getFiboLevel(fibo0, fibo100, levelValues[i], -1);
      }else{    
         linePrice = getFiboLevel(fibo100, fibo0, levelValues[i], 1);
      }
      
      HLineCreate(0, StringConcatenate("HLine",DoubleToString(fibo0),DoubleToStr(fibo100), IntegerToString(i)), 0, linePrice, fbColor, 0, 1, false, false, true, 0);
   }  
     
   return(INIT_SUCCEEDED);
  }
void OnStart()
   {

   }
void OnDeinit(const int reason)
{
   for(int i=0; i < ArraySize(levelValues); i++){
      ObjectDelete(StringConcatenate("HLine",DoubleToString(fibo0),DoubleToStr(fibo100), IntegerToString(i)));
   }

   return;
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+

double getFiboLevel( double up, double dw, double lev, int direction ){

   double ling = up - dw;
   double x = (ling) / 1.00;
   double pLev = x * lev;
   
   return ( direction < 0 ) ? NormalizeDouble( up - pLev, Digits ) : NormalizeDouble( dw + pLev, Digits ) ;

}

bool FiboLevelsCreate(const long            chart_ID=0,        // chart's ID
                      const string          name="FiboLevels", // object name
                      const int             sub_window=0,      // subwindow index 
                      datetime              time1=0,           // first point time
                      double                price1=0,          // first point price
                      datetime              time2=0,           // second point time
                      double                price2=0,          // second point price
                      const color           clr=clrRed,        // object color
                      const ENUM_LINE_STYLE style=STYLE_SOLID, // object line style
                      const int             width=1,           // object line width
                      const bool            back=false,        // in the background
                      const bool            selection=true,    // highlight to move
                      const bool            ray_right=false,   // object's continuation to the right
                      const bool            hidden=true,       // hidden in the object list
                      const long            z_order=0)         // priority for mouse click
  {
//--- set anchor points' coordinates if they are not set
   //ChangeFiboLevelsEmptyPoints(time1,price1,time2,price2);
//--- reset the error value
   ResetLastError();
//--- Create Fibonacci Retracement by the given coordinates
   if(!ObjectCreate(chart_ID,name,OBJ_FIBO,sub_window,time1,price1,time2,price2))
     {
      Print(__FUNCTION__,
            ": failed to create \"Fibonacci Retracement\"! Error code = ",GetLastError());
      return(false);
     }
//--- set color
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
//--- set line style
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
//--- set line width
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,width);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
//--- enable (true) or disable (false) the mode of highlighting the channel for moving
//--- when creating a graphical object using ObjectCreate function, the object cannot be
//--- highlighted and moved by default. Inside this method, selection parameter
//--- is true by default making it possible to highlight and move the object
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
//--- enable (true) or disable (false) the mode of continuation of the object's display to the right
   ObjectSetInteger(chart_ID,name,OBJPROP_RAY_RIGHT,ray_right);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
//--- set the priority for receiving the event of a mouse click in the chart
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution

   int levels = ArraySize(levelValues);

   ObjectSetInteger(chart_ID,name,OBJPROP_LEVELS,levels);
//--- set the properties of levels in the loop
   
   for(int i=0;i<levels;i++)
     {
      //--- level value
      ObjectSetDouble(chart_ID,name,OBJPROP_LEVELVALUE,i,levelValues[i]);
      ObjectSetInteger(chart_ID,name,OBJPROP_LEVELCOLOR,i,fbColor);

      ObjectSetString(chart_ID,name,OBJPROP_LEVELTEXT,i,DoubleToString(100*levelValues[i],1));

     }

   return(true);
  }
  
bool FiboLevelsDelete(const long   chart_ID=0,        // chart's ID
                   const string name="FiboLevels") // object name
{
//--- reset the error value
ResetLastError();
//--- delete the object
if(!ObjectDelete(chart_ID,name))
  {
   Print(__FUNCTION__,
         ": failed to delete \"Fibonacci Retracement\"! Error code = ",GetLastError());
   return(false);
  }
//--- successful execution
return(true);
}

bool HLineCreate(const long            chart_ID=0,        // chart's ID
                 const string          name="HLine",      // line name
                 const int             sub_window=0,      // subwindow index
                 double                price=0,           // line price
                 const color           clr=clrRed,        // line color
                 const ENUM_LINE_STYLE style=STYLE_SOLID, // line style
                 const int             width=1,           // line width
                 const bool            back=false,        // in the background
                 const bool            selection=true,    // highlight to move
                 const bool            hidden=true,       // hidden in the object list
                 const long            z_order=0)         // priority for mouse click
  {
//--- if the price is not set, set it at the current Bid price level
   if(!price)
      price=SymbolInfoDouble(Symbol(),SYMBOL_BID);
//--- reset the error value
   ResetLastError();
//--- create a horizontal line
   if(!ObjectCreate(chart_ID,name,OBJ_HLINE,sub_window,0,price))
     {
      Print(__FUNCTION__,
            ": failed to create a horizontal line! Error code = ",GetLastError());
      return(false);
     }
//--- set line color
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
//--- set line display style
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
//--- set line width
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,width);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
//--- enable (true) or disable (false) the mode of moving the line by mouse
//--- when creating a graphical object using ObjectCreate function, the object cannot be
//--- highlighted and moved by default. Inside this method, selection parameter
//--- is true by default making it possible to highlight and move the object
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
//--- set the priority for receiving the event of a mouse click in the chart
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  }